package testcases.scss;


import java.io.Serializable;

/**
  * A MotionEventData object is created and sent out by the CameraMD component when
  * a motion has been detected.
  *
  * A MotionEventData object is characterized by:
  * - the occurrence time of the motion
  * - the ID of the CameraMD component that has detected motion
  * - the IDs of the Switches that have been triggered by the CameraMD component. As such
  *   one is able to retrieve information about the streams that have been stored on the
  *   storage.
  * 
  * TODO: create an EventData class (Miro?)
  */
public class MotionEventData extends EventData implements Serializable {
	private long occurrenceTime;
	private String mdComponentId;
	private String[] switchIds;
	
	public MotionEventData(long occurrenceTime, String mdComponentId, String[] switchIds) {
		this.occurrenceTime = occurrenceTime;
		this.mdComponentId = mdComponentId;
		this.switchIds = switchIds;
	}
	
	public long getOccurrenceTime() {
		return occurrenceTime;
	}
	
	public String getMDComponentID() {
		return mdComponentId;
	}
	
	public String[] getSwitchesIDs() {
		return switchIds;
	}
}