package testcases.scss.motiondetection;	

public class SimpleDetector extends Detector{

    private final static int RASTER_SIZE=12;

    public SimpleDetector(){
	super();
	// Default sensitivity:
	setSensitivity(new Sensitivity(10));
    }

    public boolean detect(VideoImage image){
	if(getReferenceFrame() == null){
	    return false;
	}

	int diffValue = 0;
	byte[] pDataVideo = new byte[image.getBytesPerPixel()];
	byte[] pDataReference = new byte[image.getBytesPerPixel()];
	int nbOfChangedPixels = 0;
	int debug = 0;

//	long beginTime;
//	long processTime;
//	beginTime = System.currentTimeMillis();

	
	int yBegin = getHotspot().getY();
	int yEnd = getHotspot().getY()+getHotspot().getHeight()-1;

	int xBegin = getHotspot().getX();
	int xEnd = getHotspot().getX()+getHotspot().getWidth()-1;

	for(int y = yBegin; y < yEnd; y+=RASTER_SIZE){
	    for(int x=xBegin; x < xEnd; x+=RASTER_SIZE){
		try{

		    diffValue = 0;
		    image.pixelData(x ,y, pDataVideo);
		    getReferenceFrame().pixelData(x ,y, pDataReference);
		    

		    for(int i=0; i<pDataVideo.length; i++){
			diffValue += Math.abs(VideoImage.byte2Int(pDataVideo[i]) -
					      VideoImage.byte2Int(pDataReference[i]));
		    }

		    if(diffValue > treshHold()){ //removing noice
			nbOfChangedPixels++;
		    }		    

		}catch(ArrayIndexOutOfBoundsException aioobe){
		    // cannot really happen (unless the hotspot goes 
		    // beyond the boundaries)
		    aioobe.printStackTrace();
		    return false;
		}
	    }
	}

	//	processTime = System.currentTimeMillis()-beginTime;
	//	System.out.println("Time to process for: " + processTime);

	double changed = (double)nbOfChangedPixels;	
	changed /= getHotspot().numberOfPixels();
	changed *= (RASTER_SIZE * RASTER_SIZE);
	double sensitivity = (double)(getSensitivity().getValue());
	sensitivity /= 1000;

	return (changed >= sensitivity);
    }

    
    
    private double treshHold(){
	if(getReferenceFrame() != null){
	    return getReferenceFrame().numberOfColors()/6;
	}else{
	    return 20;
	}
    }
}
