package testcases.scss.motiondetection;

public class VideoImage{

    public final static int BW = 0;
    public final static int RGB = 1;
    public final static int RGBA = 2;

    private byte[] $videoData;
    private int $width;
    private int $height;
    private int $format;
    private int $bytesPerPixel;

    public VideoImage(byte[] videoData, int width, int height, 
		      int format, int bytesPerPixel) 
	throws InvalidImageException{
	setVideoData(videoData);
	$width = width;
	$height = height;
	$format = format;
	$bytesPerPixel = bytesPerPixel;

	if(videoData.length < (width * height * getBytesPerPixel())){
	    throw new InvalidImageException();
	}
    }

    public VideoImage(int width, int height, int format, 
		      int bytesPerPixel){
	// creates an empty video image
	$width = width;
	$height = height;
	$format = format;
	$bytesPerPixel = bytesPerPixel;

	//	$videoData = new byte[(getWidth()+1)*(getHeight()+1)*getBytesPerPixel()];
	$videoData = new byte[(getWidth())*(getHeight())*getBytesPerPixel()];
	byte b = 0;
	for(int i=0; i < getVideoData().length ; i++){
	    getVideoData()[i] = b;
	}
    }

    /*
     * This method is used to collect the data of one pixel in the image.
     *
     * @param xPos the x-position of the pixel in the image (image begins at x = 0, ends at x=getWidth()-1)
     * @param yPos the y-position of the pixel in the image (image begins at y = 0, ends at y=getHeight()-1)
     *
     * @return the byte(s) of the pixel
     *
     */
    public byte[] pixelData(int xPos, int yPos){
	try{
	    int pos;
	    // calculating first position of row at yPos
	    pos = yPos*getWidth()*getBytesPerPixel();
	    // first byte of pixel at (xPos, yPos)
	    pos += xPos*getBytesPerPixel();
	    byte[] data = new byte[getBytesPerPixel()];
	    for(int i=0; i<getBytesPerPixel(); i++){
		data[i] = getVideoData()[pos+i];
	    }
	    return data;
	}catch(ArrayIndexOutOfBoundsException aioobe){
	    // this shouldn't happen
	    aioobe.printStackTrace();
	    return null;
	}
    }

    /*
     * This method is used to collect the data of one pixel in the image.
     *
     * @param xPos the x-position of the pixel in the image (image begins at x = 0, ends at x=getWidth()-1)
     * @param yPos the y-position of the pixel in the image (image begins at y = 0, ends at y=getHeight()-1)
     *
     * @return the byte(s) of the pixel
     *
     * @require the given byte array (data) should have the correct size
     */
    public void pixelData(int xPos, int yPos, byte[] data){
	try{
	    int pos;
	    // calculating first position of row at yPos
	    pos = yPos*getWidth()*getBytesPerPixel();
	    // first byte of pixel at (xPos, yPos)
	    pos += xPos*getBytesPerPixel();

	    for(int i=0; i<getBytesPerPixel(); i++){
		data[i] = getVideoData()[pos+i];
	    }
	}catch(ArrayIndexOutOfBoundsException aioobe){
	    // this shouldn't happen
	    aioobe.printStackTrace();
	}
    }

    /**
     *  Just for optimization reasons, in case the garbage collector 
     *  doesn't support object reuse on the heap. (which is the case with kaffe)
     *
     *  precondition: data.length == $videoData.length
     * 
     */
    public void setNewVideoData(byte[] data){
	//	System.out.println("data.length: "+data.length + "   $videoData.length: "+$videoData.length);
	$videoData = data;
	/*
	  try{
	  for(int i=0; (i<$videoData.length)&&(i<data.length); i++){
	  $videoData[i] = data[i];
	  }
	  }catch(ArrayIndexOutOfBoundsException aioobe){
	  System.out.println("data.length: "+data.length + "   $videoData.length: "+$videoData.length);
	  throw aioobe;
	  }
	*/
    }
    public void setNewVideoData(VideoImage vid){
	setNewVideoData(vid.getVideoData());
    }

    public int getBytesPerPixel(){
	return $bytesPerPixel;
    }

    public int numberOfColors(){
	// deze berekening is meestal juist  ;-)  (wat bij RGBA  ?)
	return getBytesPerPixel()*256;   // 2pow8 = 256
    }

    public int numberOfPixels(){
	return getWidth()*getHeight();	
    }

    byte[] getVideoData(){
	return $videoData;
    }    

    public int getWidth(){
	return $width;
    }

    public int getHeight(){
	return $height;
    }

    public int getFormat(){
	return $format;
    }
  
    public static int byte2Int(byte b){
	int i = b;
	if(i >= 0){
	    return i;
	}else{
	    return i+256;
	}
    }  

    public Object clone(){
	try{
	    return new VideoImage(getVideoData(), getWidth(), getHeight(), 
				  getFormat(), getBytesPerPixel());
	} catch(InvalidImageException iie){
	    return new VideoImage(getWidth(), getHeight(), 
				  getFormat(), getBytesPerPixel());
	}
    }

    private void setVideoData(byte[] vd){
	$videoData = new byte[vd.length];
	for(int i=0; i<vd.length; i++){
	    $videoData[i] = vd[i];
	}
    }
}
