package testcases.scss.motiondetection.test;

import testcases.scss.motiondetection.*;

import thesis.application.imageclient.*;
import thesis.dc1394.*;

import java.io.*;
import javax.sound.sampled.*;
import javax.sound.sampled.spi.*;

public class CameraMDTest implements ImageListener {

    private boolean $detecting = false;
    private int $width;
    private int $height;
    private int $colorFormat;
    private int $bytesPerPixel;
    private boolean $evolvable = true; // evolvable or static
    //    private VideoImage $referenceFrame;
    private int $frameRate = 2;  // in frames per second
    private long $timePreviousFrame = 0;
    private boolean $zooming = false;
    private Detector $detector;
    private long $idleTime = 0L;
    private int $sensitivityFactor = DETECT_CHANGES; //default value

//    private Hotspot $hotspot;

    private final static int DETECT_NO_CHANGE = 1000;
    private final static int DETECT_LARGE_CHANGES = 50;
    private final static int DETECT_CHANGES = 18;
    private final static int DETECT_SMALL_CHANGES = 10;
    private final static int DETECT_EVERY_CHANGE = 5;
  
    private Clip $clip;

    public CameraMDTest(){
	setDetector(new SimpleDetector());
	getDetector().setSensitivity(new Sensitivity($sensitivityFactor));
	setEvolvable(true);
	setBytesPerPixel(3);
	setHotspot(new Hotspot(0, 0, 110, 239));
	setFrameRate(3);
	setWidth(320);
	setHeight(240);
	$clip = SoundPlayer.loadSound(new File("intalt8.wav"));
    }

    public void newIm(byte[] videoImage){	
	if(initialized() && atFrameRate() && !zooming()){ // maybe we should use the state pattern ?
	    try{
		VideoImage image = new VideoImage(videoImage, 
						  getWidth(), 
						  getHeight(), 
						  getColorFormat(),
						  getBytesPerPixel());

//		System.out.println("Video Image constructed");
		
		if(!evolvable() && getDetector().getReferenceFrame()==null){
		    getDetector().setReferenceFrame(image);
		}
		
		// detecting motion:
		if(getDetector().detect(image)){
		    if(!detecting()){
			detected();
			setDetecting(true);
		    }
		}else{
		    // there is no motion detected:
		    //		    System.out.print("*");
		    if(detecting()){
			// stop detecting when we don't detect motion 
			// for 2 seconds or more:
			if($idleTime == 0){
			    $idleTime = System.currentTimeMillis();
			}else if(System.currentTimeMillis()-$idleTime > 2000){
			    setDetecting(false);
			    $idleTime = 0;
			}
		    }
		}
		
		if(evolvable()){
		    getDetector().setReferenceFrame(image);
		}
	    }catch(InvalidImageException iie){
		
	    }
	    
	}
    }
    /* ***********************************
     *  implementing ImageListener: 
     *
     * ***********************************
     */

    public void newImage(CameraImage im){
	newIm(im.getData());
    }

    public void close(){
	return;
    }

    /* ***********************************
     */

    /* ***********************************
     *  private hulpmethodes 
     *
     * ***********************************
     */

    private boolean detecting(){
	return $detecting;
    }

    private void setDetecting(boolean d){
	$detecting = d;
    }

    private int numberOfColors(){
	// deze berekening is meestal juist  ;-)  (wat bij RGBA ?)
	return getBytesPerPixel()*256;   // 2pow8 = 256
    }


    private boolean initialized(){
	// is onze component voldoende geinitializeerd om aan motion detection
	// te doen ?
	return (getBytesPerPixel()>0);
    }


    /*
     * We work at a limited framerate (see specs).
     * 
     * @return wether we are on time or too soon
     */
    private boolean atFrameRate(){
	long current = System.currentTimeMillis();
	if((current - $timePreviousFrame) >= 1000/getFrameRate()){
	    $timePreviousFrame = current;
	    return true;
	}
	return false;
    }  

    private void setDetector(Detector detector){
	$detector = detector;
    }

    private Detector getDetector(){
	return $detector;
    }

    private boolean zooming(){
	return $zooming;
    }

    private void setZooming(boolean zooming){
	$zooming = zooming;
    }

    private Hotspot getHotspot(){
	return getDetector().getHotspot();
    }

    private void setHotspot(Hotspot hotspot){
	getDetector().setHotspot(hotspot);
    }
    private void setFrameRate(int frameRate){
	$frameRate = frameRate;
    }
    private int getFrameRate(){
	return $frameRate;
    }
    private void setBytesPerPixel(int bytesPerPixel){
	$bytesPerPixel = bytesPerPixel;
    }
    private int getBytesPerPixel(){
	return $bytesPerPixel;
    }
    private void setEvolvable(boolean evolvable){
	$evolvable = evolvable;
    }
    private boolean evolvable(){
	return $evolvable;
    }
    private int numberOfPixels(){
	return getWidth()*getHeight();
    }
    private int getWidth(){
	return $width;
    }

    private int getHeight(){
	return $height;
    }

    private void setWidth(int width){
	$width = width;
    }

    private void setHeight(int height){
	$height = height;
    }
    
    private int getColorFormat(){
	return $colorFormat;
    }

    private void setColorFormat(int colorFormat){
	$colorFormat = colorFormat;
    }

    private void detected(){
	if($clip != null){
	    SoundPlayer.playSound($clip);
	}
    }

}
