#define JNIEXPORT

#define BYTES_PER_PIXEL 3 // 1, 3 of 4

#include <jni.h>
#include "testcases_scss_Sony.h"

#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>

#define AUTOMATIC 0x83000000

static raw1394handle_t handle;
static dc1394_cameracapture camera;
static char * buffer;
static size;
static aantal_pixels;

JNIEXPORT void JNICALL Java_testcases_scss_Sony_init(JNIEnv *env, jobject obj, jint format, jint mode, jint speed, jint framerate) {
	int i,num;
	dc1394bool_t OK=0;
	dc1394bool_t OK_dezenode=0;

	// we gebruiken FireWire-bus 0
	handle = dc1394_create_handle(0);
	if (handle==NULL) {
		printf("Unable to aquire a raw1394 handle\n");
		exit(1);
	}

	// Bepaal aantal nodes op de bus
	num = raw1394_get_nodecount(handle);  
	if (num<1) { // geen node???
		printf("No nodes!\n");
		raw1394_destroy_handle(handle);
		exit(1);
	}

	// Test of we minstens 1 camera hebben
	OK=0;
	for (i=0; i<num; i++){
		dc1394_is_camera(handle, i, &OK_dezenode);
		OK |= OK_dezenode;
	}
	if (!OK){
		printf("Sorry, geen camera gevonden op de bus!\n");
		raw1394_destroy_handle(handle);
		exit(1);
	}

	// Reset bus tot de camera knoop 0 is
	OK=0;
	while (!OK){
		dc1394_is_camera(handle, 0, &OK);
		if (!OK) raw1394_reset_bus(handle);
	}

	// We hebben een camera op node 0 => initialisatie
	dc1394_init_camera(handle, 0);

	if (dc1394_dma_setup_capture(handle,0,0, format, mode, speed,framerate, 8, &camera)!=DC1394_SUCCESS) {
		printf("Unable to setup camera!\n");
		dc1394_release_camera(handle, &camera);
		raw1394_destroy_handle(handle);
		exit(1);
	}

	aantal_pixels=camera.frame_width*camera.frame_height;
	size=camera.frame_width*camera.frame_height*BYTES_PER_PIXEL;

	buffer=(char*)malloc(size*1);
	if (!buffer) {
		printf("Could not allocate framebuffer!\n");
		exit(1);
	}

	// Zet zo veel mogelijk features in automatische mode
	SetFeatureValue(handle, 0 /*node*/, 0x820 /*REG_CAMERA_GAIN*/,          AUTOMATIC);  // auto gain control

}

JNIEXPORT void JNICALL Java_testcases_scss_Sony_startiso(JNIEnv *env, jobject obj) {
	if (dc1394_start_iso_transmission(handle, 0)!=DC1394_SUCCESS){
		dc1394_release_camera(handle, &camera);
		raw1394_destroy_handle(handle);
	}
}

JNIEXPORT void JNICALL Java_testcases_scss_Sony_stopiso(JNIEnv *env, jobject obj) {
	if (dc1394_stop_iso_transmission(handle, 0)!=DC1394_SUCCESS){
		dc1394_release_camera(handle, &camera);
		raw1394_destroy_handle(handle);
	}
}

JNIEXPORT void JNICALL Java_testcases_scss_Sony_singlecapture (JNIEnv *env, jobject obj, jint index, jboolean bw){
	if (dc1394_dma_single_capture(&camera)!=DC1394_SUCCESS) {
		dc1394_release_camera(handle, &camera);
		raw1394_destroy_handle(handle);
	} 

	if (bw)
	  yuv422tobw(/*source*/camera.capture_buffer, /*dest*/buffer+index*size, aantal_pixels);
	else
	  yuv422torgb(/*source*/camera.capture_buffer, /*dest*/buffer+index*size, aantal_pixels);

//	yuv422torgba(/*source*/camera.capture_buffer, /*dest*/buffer+index*size, size/4);

	if (dc1394_dma_done_with_buffer(&camera)!=DC1394_SUCCESS){
		printf("couldn't clear the DMA buffer\n");
	}

}

JNIEXPORT void JNICALL Java_testcases_scss_Sony_getimage (JNIEnv *env, jobject obj, jbyteArray image, jint offset, jboolean bw, jint beeld){
	(*env)->SetByteArrayRegion(env, image, 0/*offset*/, aantal_pixels*(bw?1:3), (jbyte*)buffer+beeld*size);
}

JNIEXPORT jint JNICALL Java_testcases_scss_Sony_setfeature(JNIEnv *env, jobject obj, jlong feature, jint value){
	return SetFeatureValue(handle, 0 /*node*/, feature , value); 
}

JNIEXPORT jint JNICALL Java_testcases_scss_Sony_getfeature(JNIEnv *env, jobject obj, jlong feature){
	int value;
	GetFeatureValue(handle, 0 /*node*/, feature, &value);
	return value;
}

