package testcases.scss.util;

import java.awt.Image;
import java.awt.image.MemoryImageSource;
import java.awt.Canvas;

/**
 * Canvas image creation strategy.
 *
 *  Represents the default strategy for creating images (java.awt.Image)
 *  from byte arrays. This strategy uses an java.awt.image.MemoryImageSource
 *  and a java.awt.Canvas object.
 *
 * @author Dennis Wagelaar
 * @version 23 October 2002
 */

public class CanvasImageCreationStrategy implements ImageCreationStrategy {

	/**
	 * Creates an Image from a byte array. Assumes a grayscale
	 * image (8 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createBWImage(int width, int height, byte[] image) {
		int pixel;
		int[] pixels = new int[width * height];
		MemoryImageSource source;
		Canvas c = new Canvas();
		// BW
		for (int i = 0; i < width * height; i++) {
			pixel = image[i];
			pixel &= 0xFF;
			pixels[i] = (255 << 24) | (pixel << 16) | (pixel << 8) | pixel;
		}
		source = new MemoryImageSource (width, height, pixels, 0, width);
		return c.createImage(source);
	} //createBWImage

	/**
	 * Creates an Image from a byte array. Assumes an RGB
	 * image (24 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createRGBImage(int width, int height, byte[] image) {
		int r, g, b;
		int[] pixels = new int[width * height];
		MemoryImageSource source;
		Canvas c = new Canvas();
		// Normal RGB
		for (int i = 0; i < width * height; i++) {
			r = image[i * 3];
			g = image[i * 3 + 1];
			b = image[i * 3 + 2];
			r &= 0xFF;
			g &= 0xFF;
			b &= 0xFF;
			pixels[i] = (255 << 24) | (r << 16) | (g << 8) | b;
		}
		source = new MemoryImageSource (width, height, pixels, 0, width);
		return c.createImage(source);
	} //createRGBImage

} //class CanvasImageCreationStrategy
