package testcases.scss.util;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

import java.io.File;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import java.awt.Image;
import javax.swing.ImageIcon;

/**
 * Database connectivity class.
 *
 *  Connects to the MySQL database "storage" and handles queries
 *  to the database.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @version 5 November 2002
 */

public class DatabaseListener {

	////////////////////////////////////////////////////////////////////////
	// Constants
	////////////////////////////////////////////////////////////////////////

	private static final String JDBC_URL =
		"jdbc:mysql://localhost/storage?user=seescoa&password=seescoa";
	private static final String IMG_PATH = "testcases/scss/storage/imagedatabase/";

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private Connection con;
	private Statement stmt;
	//private int precision = 200;

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs a DatabaseListener.
	 * @author Miro Casanova
	 */
	public DatabaseListener() {
	} //DatabaseListener

	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Inserts an event in the database.
	 * @param ev The event to insert
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void insertEvent(EventData ev)
	{
		/* insert into image
		   values (imID,cmID,tm,pth) */

		int count = 0, ms;
		String msg, evID, cmID, tm;

		tm   = ev.getTime();
		ms   = ev.getMilliSec();
		cmID = ev.getComponentID().getID();
		msg  = ev.getComment();
		evID = ev.getID().getID();

		String query = "INSERT INTO event "+
			"VALUES ('"+evID+"','"+cmID+"','"+tm+"',"+ms+",'"+msg+"')";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			stmt.executeUpdate(query);
			stmt.close();
			con.close();
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of images retrieved");
		}
	} //insertEvent

	/**
	 * Inserts an alarm into the database.
	 * @param al The alarm to insert
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void insertAlarm(AlarmData al)
	{
		/* insert into image
		   values (imID,cmID,tm,pth) */

		int count = 0, ms;
		String msg, alID, cmID, tm;

		tm   = al.getTime();
		ms   = al.getMilliSec();
		cmID = al.getCameraID().getID();
		msg  = al.getDescription();
		alID = al.getID().getID();

		String query = "INSERT INTO alarm "+
			"VALUES ('"+alID+"','"+cmID+"','"+tm+"',"+ms+",'"+msg+"')";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			stmt.executeUpdate(query);
			stmt.close();
			con.close();
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of images retrieved");
		}
	} //insertAlarm

	/**
	 * Inserts an image in the database.
	 * @param im The image
	 * @author Miro Casanova
	 */
	public void insertImage(StorageVideoImage im)
	{

		/* insert into image
		   values (imID,cmID,tm,pth) */

		int count = 0, ms;
		String imID, tm, cmID;
		File path;

		tm   = im.getTime();
		ms   = im.getMilliSec();
		cmID = im.getCameraID().getID();
		imID = im.getID().getID();
		path = this.createFileName(imID);
		this.createFile(path, im.getImage());


		String query = "INSERT INTO image "+
			"VALUES ('"+imID+"','"+cmID+"','"+tm+"',"+ms+",'"+path+"')";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			stmt.executeUpdate(query);
			stmt.close();
			con.close();
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of images retrieved");
		}
	} //insertImage

	/**
	 * Queries the database for an image.
	 * @param time The date/time
	 * @param camid The camera ID
	 * @return The retrieved image
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public StorageVideoImage queryImage(TimeData time, ID camid)
	{
		/* select path, time, millisec from image
		   where time = t and cameraID = cid */

		StorageVideoImage image = null;
		int count = 0, i = 0;
		String ts = time.getTime();
		String tstamp = time.getTimestamp();
		String id = camid.getID();
		int min = 0, max = 0;
		int ms = time.getMilliSec();
		java.util.Date tm = time.getTimeAsDate();
		String path = new String();

		/* -- Old query --
		if( ms >= 0 )
		{
			min = ( ms > precision )? ms - precision : 0;
			max = ( ms + precision < 999 )? ms + precision: 999;
		}
		String query="";
		if( ms>=0 )
			query = "SELECT path, millisec FROM image i "+
				"WHERE i.time = '"+ts+"' AND (i.millisec BETWEEN "+min+" AND "+max+") AND i.cameraID = '"+id+"' "+
				"ORDER BY i.time LIMIT 1";
		else
			query = "SELECT path, millisec FROM image i "+
				"WHERE i.time = '"+ts+"' AND i.cameraID = '"+id+"' "+
				"ORDER BY i.time LIMIT 1";
		-- End old query -- */

		/*
		// searches for the first image of the given camera since the given time
		String query = "SELECT path, time, millisec FROM image " +
			"WHERE cameraID = '" + id + "' AND (time > '" + ts +
			"' OR (time = '" + ts + "' AND millisec >= " + ms + ")) " +
			"ORDER BY time, millisec LIMIT 1";
		*/
		// searches for the image of the given camera that is closest to the given time
		String query = "SELECT path, time, millisec, " +
			"ABS(('" + tstamp + "' - time) * 1000 + (" + ms + " - millisec)) AS dtime " +
			"FROM image " +
			"WHERE cameraID = '" + id + "' " +
			"ORDER BY dtime LIMIT 1";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			while( rs.next() )
			{
				path = rs.getString("path");
				tm   = rs.getTimestamp("time");
				ms   = rs.getInt("millisec");
				i++;
			}
			if( i==0 )
			{
				System.out.println("No image found");
				throw new Exception();
			}
			stmt.close();
			con.close();

			// image exists, so retrieve from file
			time.setTimeAsDate(tm); // update date/time
			time.setMilliSec(ms); // update millisecond part
			image = new StorageVideoImage(time, camid, readImageFile(new File(path)));
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of images retrieved");
		}

		return image;
	} //queryImage

	/**
	 * Queries the database for an event.
	 * @param time The date/time
	 * @param compid The component ID
	 * @return The event data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public EventData queryEvent(TimeData time, ID compid)
	{
		/* select message, time, millisec from event
		   where time = t and componentID = cid */

		EventData result = null;
		int count = 0, i = 0;
		int min = 0, max = 0;
		int ms = time.getMilliSec();
		String ts = time.getTime();
		String tstamp = time.getTimestamp();
		String id = compid.getID();
		java.util.Date tm = time.getTimeAsDate();
		String message = new String();

		/* -- Old query --
		if( ms >=0 )
		{
			min = ( ms > precision )? ms - precision : 0;
			max = ( ms + precision < 999 )? ms + precision: 999;
		}
		String query;
		if( ms >=0 )
			query = "SELECT message, millisec FROM event e "+
				"WHERE e.time = '"+ts+"' AND (e.millisec BETWEEN "+min+" AND "+max+") AND e.componentID = '"+id+"' "+
				"ORDER BY e.time LIMIT 1";
		else
			query = "SELECT message, millisec FROM event e "+
				"WHERE e.time = '"+ts+"' AND e.componentID = '"+id+"' "+
				"ORDER BY e.time LIMIT 1";
		-- End old query -- */

		/*
		// searches for the first event of the given component since the given time
		String query = "SELECT message, time, millisec FROM event " +
			"WHERE componentID = '" + id + "' AND (time > '" + ts +
			"' OR (time = '" + ts + "' AND millisec >= " + ms + ")) " +
			"ORDER BY time, millisec LIMIT 1";
		*/
		// searches for the event of the given component that is closest to the given time
		String query = "SELECT message, time, millisec, " +
			"ABS(('" + tstamp + "' - time) * 1000 + (" + ms + " - millisec)) AS dtime " +
			"FROM event " +
			"WHERE componentID = '" + id + "' " +
			"ORDER BY dtime LIMIT 1";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			while( rs.next() ) {
				message = rs.getString("message");
				tm      = rs.getTimestamp("time");
				ms      = rs.getInt("millisec");
				i++;
			}
			if( i==0 )
				throw new Exception();
			stmt.close();
			con.close();

			// Event exists
			time.setTimeAsDate(tm); // update date/time
			time.setMilliSec(ms); // update millisecond part
			result = new EventData(time, compid, message);
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of events retrieved");
		}

		return result;
	} //queryEvent

	/**
	 * Queries the database for an event.
	 * @param time The date/time
	 * @param camid The camera ID
	 * @return The alarm data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public AlarmData queryAlarm(TimeData time, ID camid)
	{
		/* select message, time, millisec from event
		   where time = t and componentID = cid */

		AlarmData result = null;
		int count = 0, i = 0;
		int min = 0, max = 0;
		String ts = time.getTime();
		String tstamp = time.getTimestamp();
		String id = camid.getID();
		int ms = time.getMilliSec();
		java.util.Date tm = time.getTimeAsDate();
		String message = new String();

		/* -- Old query --
		if( ms>= 0 )
		{
			min = ( ms > precision )? ms - precision : 0;
			max = ( ms + precision < 999 )? ms + precision: 999;
		}
		String query="";
		if( ms>=0 )
			query = "SELECT message, millisec FROM alarm a "+
				"WHERE a.time = '"+ts+"' AND (a.millisec BETWEEN "+min+" AND "+max+") AND a.cameraID = '"+id+"' "+
				"ORDER BY a.time LIMIT 1" ;
		else
			query = "SELECT message, millisec FROM alarm a "+
				"WHERE a.time = '"+ts+"' AND a.cameraID = '"+id+"' "+
				"ORDER BY a.time LIMIT 1" ;
		-- End old query -- */

		/*
		// searches for the first alarm of the given component since the given time
		String query = "SELECT message, time, millisec FROM alarm " +
			"WHERE cameraID = '" + id + "' AND (time > '" + ts +
			"' OR (time = '" + ts + "' AND millisec >= " + ms + ")) " +
			"ORDER BY time, millisec LIMIT 1";
		*/
		// searches for the image of the given camera that is closest to the given time
		String query = "SELECT message, time, millisec, " +
			"ABS(('" + tstamp + "' - time) * 1000 + (" + ms + " - millisec)) AS dtime " +
			"FROM alarm " +
			"WHERE cameraID = '" + id + "' " +
			"ORDER BY dtime LIMIT 1";

		try {
			Class.forName("org.gjt.mm.mysql.Driver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			con = DriverManager.getConnection(JDBC_URL);
			stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			while( rs.next() ) {
				message = rs.getString("message");
				tm      = rs.getTimestamp("time");
				ms      = rs.getInt("millisec");
				i++;
			}
			if( i==0 )
				throw new Exception();
			stmt.close();
			con.close();

			// Alarm exists
			time.setTimeAsDate(tm); // update date/time
			time.setMilliSec(ms); // update millisecond part
			result = new AlarmData(time, camid, message);
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		} catch (Exception ex) {
			System.out.println("Exception: Erroneous amount of events retrieved");
		}

		return result;
	} //queryAlarm

	////////////////////////////////////////////////////////////////////////
	// Private methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Creates a file using the given path and content.
	 * @param path The filename path
	 * @param im The image file content
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	private void createFile(File path, Image im) {
		ImageIcon imageIcon = null;

		try {
			// create output streams
			FileOutputStream fileOut = new FileOutputStream(path);
			ObjectOutputStream objectOut = new ObjectOutputStream(fileOut);
			// write the image object
			imageIcon = new ImageIcon(im);
			objectOut.writeObject(imageIcon);
			// flush & close streams
			objectOut.flush();
			objectOut.reset();
			objectOut.close();
			fileOut.close();
			// flush Image
			im.flush();
		} catch( IOException e ) {
			System.err.println("Problems while creating an image (" + e.getMessage() + ")");
		}
	} //createFile

	/**
	 * Reads an Image object from the specified file.
	 * @param path The path to the file
	 * @return The image contained in the file or null
	 * @author Dennis Wagelaar
	 */
	private Image readImageFile(File path) {
		ImageIcon imageIcon = null;
		Image image = null;

		try {
			// create input streams
			FileInputStream fileIn = new FileInputStream(path);
			ObjectInputStream objectIn = new ObjectInputStream(fileIn);
			// read image object from file
			imageIcon = (javax.swing.ImageIcon) objectIn.readObject();
			image = imageIcon.getImage();
			// close streams
			objectIn.close();
			fileIn.close();
		} catch( IOException e ) {
			System.err.println("Problems while reading an image from file (" + e.getMessage() + ")");
		} catch ( ClassNotFoundException ce ) {
			System.err.println("Unknown class contained in the image file (" + ce.getMessage() + ")");
		}

		return image;
	} //readImageFile

	/**
	 * Creates a image filename from an identifier.
	 * @param id The ID
	 * @return The filename object
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	private File createFileName(String id)
	{
		return new File(IMG_PATH + id + ".img");
	} //createFileName

} //class DatabaseListener
