package testcases.scss.util;

import java.awt.Image;
import java.io.Serializable;

/**
 * Image creation strategy interface.
 *
 *  Interface for a strategy for creating images (java.awt.Image)
 *  from byte arrays.
 *
 * @author Dennis Wagelaar
 * @version 23 October 2002
 */

public interface ImageCreationStrategy
	extends Serializable {

	/**
	 * Creates an Image from a byte array. Assumes a grayscale
	 * image (8 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createBWImage(int width, int height, byte[] image);

	/**
	 * Creates an Image from a byte array. Assumes an RGB
	 * image (24 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createRGBImage(int width, int height, byte[] image);

} //interface ImageCreationStrategy
