package testcases.scss.util;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.StringBufferInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.net.URL;
import java.net.MalformedURLException;

//SAX exceptions needed for dom parsing
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

//DOM Interfaces
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Text;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.apache.xerces.framework.XMLParser;
import org.xml.sax.InputSource;


/** This class takes a couple of UiDescriptions and merges them into one.
 *
 */

public class UiMerger{

	private String[] $uiStrings = null;

	private PrintWriter $out = null;
	private static String NEW_UIDESCRIPTION = "mergeduidescription.xml";

	public UiMerger(String[] uiStrings){
		$uiStrings = uiStrings;
		//		System.out.println("Nerger instantiated for: " + uiStrings.length + " uidescriptions");
	}

	public String mergeUiDescriptions(){
	    //		System.out.println("in mergeUiDescriptions");

		try{
			$out = new PrintWriter(new FileOutputStream(NEW_UIDESCRIPTION));
		}catch(FileNotFoundException fnfe){
			fnfe.printStackTrace();
		}

		//parse every xml description and put the roots  in an Element[] array
		Element[] rootNodes = new Element[2];
		
		XMLParser parser = new org.apache.xerces.parsers.DOMParser();

		for(int i=0; i<2; i++){
			try{
				parser.parse(new InputSource(new StringBufferInputStream($uiStrings[i])));
				rootNodes[i] = (((org.apache.xerces.parsers.DOMParser)parser).getDocument()).getDocumentElement();
			}catch(IOException ioe){
				ioe.printStackTrace();
			}catch(SAXException se){
				se.printStackTrace();
			}
		}

		Document doc = null;

		try{
			//creating the new document to embed the other uidescriptions in
			doc = (Document)Class.forName("org.apache.xerces.dom.DocumentImpl").newInstance();
		}catch(java.lang.ClassNotFoundException cnfe){
			cnfe.printStackTrace();
		}catch(java.lang.InstantiationException iex){
			iex.printStackTrace();
		}catch(java.lang.IllegalAccessException iaex){
			iaex.printStackTrace();
		}

		if( doc !=  null){
			Element root = doc.createElement("ui");
			Element title = doc.createElement("title");
			title.appendChild(doc.createTextNode("Merged Camera + Motion Detection"));
			root.appendChild(title);
			
			Element group = doc.createElement("group");
			group.setAttribute("name", "MDCamGroup");
			group.setAttribute("rows", "1");
			group.setAttribute("columns", "2");
			group.setAttribute("splittable", "yes");

			root.appendChild(group);
			doc.appendChild(root);

			//Put all the first groups of the old UiDescriptions into the new
			//UiDescription

			for( int i=0; i<rootNodes.length; i++){
				Element rootElement = rootNodes[i];
				NodeList children = rootElement.getChildNodes();

				boolean firstGroupEncountered = false;

				int j = 0;
				while(!firstGroupEncountered){
					Node node = children.item(j);
					if(node.getNodeName().equals("group")){
						firstGroupEncountered = true;
						group.appendChild(doc.importNode(node, true));
					}
					j++;
				}
			}

			writeToDisk(doc);
			$out.flush();
			$out.close();
			
		}
		
		InputStream is = null;
		//Open the file, make it an InputStream and return it
		try{
			is = new FileInputStream(NEW_UIDESCRIPTION);
		}catch(FileNotFoundException fnfe){
			fnfe.printStackTrace();
		}
		
		
		//inputstream to string
		//
		return inputStreamToString(is);
				
	}

	private String inputStreamToString(InputStream is){
		StringBuffer uiDesc = null;
		try{
			BufferedReader in = new BufferedReader(new InputStreamReader(is));
			uiDesc = new StringBuffer();
			String xmlLine = in.readLine();
			while( xmlLine != null){
				uiDesc.append(xmlLine);
				xmlLine = in.readLine();
		    	}
		}catch(IOException ioe){
			ioe.printStackTrace();
		}

		return uiDesc.toString();
	}

	//Print the new DOMTree to a file
	 private void writeToDisk(Node node)
    {
		int type = node.getNodeType();
        if(type == Node.DOCUMENT_NODE){
              $out.println("<?xml version=\"1.0\" encoding=\""+
                                "UTF-8" + "\"?>");
        }else if(type==Node.ELEMENT_NODE){
                $out.print('<' + node.getNodeName() );
                NamedNodeMap nnm = node.getAttributes();
                if(nnm != null )
                {
                    int len = nnm.getLength() ;
                    Attr attr;
                    for ( int i = 0; i < len; i++ )
                    {
                        attr = (Attr)nnm.item(i);
                        $out.print(' '
                             + attr.getNodeName()
                             + "=\""
                             + attr.getNodeValue()
                             +  '"' );
                    }
                }
                $out.print('>');
	}else if(type==Node.ENTITY_REFERENCE_NODE){
               $out.print('&' + node.getNodeName() + ';' );

        }else if(type == Node.CDATA_SECTION_NODE){
               $out.print( "<![CDATA["
                     	+ node.getNodeValue()
                            + "]]>" );
        }else if(type == Node.TEXT_NODE){
               $out.print(node.getNodeValue());
        }else if(type == Node.PROCESSING_INSTRUCTION_NODE){
               $out.print("<?" + node.getNodeName() ) ;
               String data = node.getNodeValue();
               if ( data != null && data.length() > 0 ) {
                   $out.print(' ');
                   $out.print(data);
               }
               $out.println("?>");
         }

        //recurse
        for(Node child = node.getFirstChild(); child != null; child = child.getNextSibling()){
		writeToDisk(child);
        }

        //without this the ending tags will miss
        if ( type == Node.ELEMENT_NODE )
        {
            $out.print("</" + node.getNodeName() + ">");
        }

	//			System.out.println("Writen the new file to disk");

    }//end of walk

}
