package testcases.scss.util.test;

/**
 * Alarm data class for SEESCOA.
 *
 *  AlarmData stores a date/time in a precision of milliseconds, a
 *  camera ID and a description.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @version 14 October 2002
 */

public class AlarmData extends TimeData {

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private ID $camid = null;
	private String $descr = null;

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs an AlarmData.
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	AlarmData() {
		super();
	} //AlarmData

	/**
	 * Constructs an AlarmData.
	 * @param time The time data
	 * @param camid The component ID
	 * @param descr The alarm description
	 * @author Dennis Wagelaar
	 */
	AlarmData(TimeData time, ID camid, String descr) {
		super(time);
		this.setCameraID(camid);
		this.setDescription(descr);
	} //AlarmData

	/**
	 * Constructs an AlarmData.
	 * @param year The year (e.g. 2002)
	 * @param month The month (e.g. 12)
	 * @param day The day of the month (e.g. 31)
	 * @param hour The hour (e.g. 23)
	 * @param minute The minute (e.g. 59)
	 * @param second The second (e.g. 59)
	 * @param msecond The amount of milliseconds (e.g. 999)
	 * @param camid The component ID
	 * @param descr The alarm description
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	AlarmData(int year, int month, int day, int hour, int minute, int second,
			int msecond, ID camid, String descr) {
		super(year, month, day, hour, minute, second, msecond);
		this.setCameraID(camid);
		this.setDescription(descr);
	} //AlarmData

	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the camera ID attribute.
	 * @param camid The camera ID
	 * @author Dennis Wagelaar
	 */
	public void setCameraID(ID camid) {
		this.$camid = camid;
	} //setCameraID

	/**
	 * Gets the camera ID attribute.
	 * @return The camera ID
	 * @author Dennis Wagelaar
	 */
	public ID getCameraID() {
		return this.$camid;
	} //getCameraID

	/**
	 * Sets the description attribute.
	 * @param descr The description
	 * @author Dennis Wagelaar
	 */
	public void setDescription(String descr)
	{
		this.$descr = descr;
	} //setDescription

	/**
	 * Gets the description attribute.
	 * @return The description
	 * @author Dennis Wagelaar
	 */
	public String getDescription()
	{
    	return this.$descr;
	} //getDescription

	/**
	 * Generates an ID for this object from the camera ID attribute
	 * and the timestamp.
	 * @return The ID for this object
	 * @author Dennis Wagelaar
	 */
	public ID getID() {
		return new ID(this.getCameraID().getID() + this.getTimestamp() + this.getMilliSec());
	} //getID

} //class AlarmData
