package testcases.scss.util.test;

/**
 * Event data class for SEESCOA.
 *
 *  EventData stores a date/time in a precision of milliseconds, a
 *  component ID and a comment.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @version 14 October 2002
 */

public class EventData extends TimeData {

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private ID $compid = null;
	private String $comment = null;

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs an EventData.
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	EventData() {
		super();
	} //EventData

	/**
	 * Constructs an EventData.
	 * @param time The time data
	 * @param compid The component ID
	 * @param comment The event comment
	 * @author Dennis Wagelaar
	 */
	EventData(TimeData time, ID compid, String comment) {
		super(time);
		this.setComment(comment);
		this.setComponentID(compid);
	} //EventData

	/**
	 * Constructs an EventData.
	 * @param year The year (e.g. 2002)
	 * @param month The month (e.g. 12)
	 * @param day The day of the month (e.g. 31)
	 * @param hour The hour (e.g. 23)
	 * @param minute The minute (e.g. 59)
	 * @param second The second (e.g. 59)
	 * @param msecond The amount of milliseconds (e.g. 999)
	 * @param compid The component ID
	 * @param comment The event comment
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	EventData(int year, int month, int day, int hour, int minute, int second,
			int msecond, ID compid, String comment) {
		super(year, month, day, hour, minute, second, msecond);
		this.setComment(comment);
		this.setComponentID(compid);
	} //EventData

	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the ccomponent ID attribute.
	 * @param compid The component ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void setComponentID(ID compid)
	{
		this.$compid = compid;
	} //setComponentID

	/**
	 * Sets the component ID attribute.
	 * @return The component ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public ID getComponentID()
	{
		return this.$compid;
	} //getComponentID

	/**
	 * Sets the comment attribute.
	 * @param comment The comment
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void setComment(String comment)
	{
		this.$comment = comment;
	} //setComment

	/**
	 * Gets the comment attribute.
	 * @return The comment
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public String getComment()
	{
		return this.$comment;
	} //getComment

	/**
	 * Generates an ID for this object from the component ID attribute
	 * and the timestamp.
	 * @return The ID for this object
	 * @author Dennis Wagelaar
	 */
	public ID getID() {
		return new ID(this.getComponentID().getID() + this.getTimestamp() + this.getMilliSec());
	} //getID

} //class EventData
