package testcases.scss.util.test;

import java.awt.Image;

/**
 * Video image class returned by the StorageController.
 *
 *  This class contains all data on a video image stored in the
 *  database. It stores a date/time in a precision of milliseconds,
 *  a camera ID and an image.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @version 15 October 2002
 */

public class StorageVideoImage extends TimeData {

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private ID $camid = null;
	private Image $image = null;

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs a StorageVideoImage.
	 * @author Miro Casanova
	 */
	StorageVideoImage() {
		super();
	} //StorageVideoImage

	/**
	 * Constructs a StorageVideoImage.
	 * @param time The time data for the image
	 * @param camid The camera ID
	 * @param image The image data
	 * @author Dennis Wagelaar
	 */
	StorageVideoImage(TimeData time, ID camid, Image image) {
		super(time);
		this.setCameraID(camid);
		this.setImage(image);
	} //StorageVideoImage

	/**
	 * Constructs a StorageVideoImage.
	 * @param year The year (e.g. 2002)
	 * @param month The month (e.g. 12)
	 * @param day The day of the month (e.g. 31)
	 * @param hour The hour (e.g. 23)
	 * @param minute The minute (e.g. 59)
	 * @param second The second (e.g. 59)
	 * @param msecond The amount of milliseconds (e.g. 999)
	 * @param camid The camera ID
	 * @param image The image data
	 * @author Dennis Wagelaar
	 */
	StorageVideoImage(int year, int month, int day, int hour, int minute,
			int second, int msecond, ID camid, Image image) {
		super(year, month, day, hour, minute, second, msecond);
		this.setCameraID(camid);
		this.setImage(image);
	} //StorageVideoImage

	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the image data attribute.
	 * @param i The image data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void setImage(Image i) {
		this.$image = i;
	} //setImage

	/**
	 * Gets the image data attribute.
	 * @return The image data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public Image getImage() {
		return this.$image;
	} //getImage

	/**
	 * Sets the camera ID attribute.
	 * @param camid The camera ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void setCameraID(ID camid) {
		this.$camid = camid;
	} //setCameraID

	/**
	 * Gets the camera ID attribute.
	 * @return The camera ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public ID getCameraID() {
		return this.$camid;
	} //getCameraID

	/**
	 * Generates an ID for this object from the camera ID attribute
	 * and the timestamp.
	 * @return The ID for this object
	 * @author Dennis Wagelaar
	 */
	public ID getID() {
		return new ID(this.getCameraID().getID() + "-" + this.getTimestamp() + "-" + this.getMilliSec());
	} //getID

} //class StorageVideoImage
